/* ====================================================================
 * Copyright (c) 1999 GlobeCom GCAB AB.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by GlobeCom GCAB AB
 *    <http://www.globecom.net/> for use in GlobeCom Jukebox."
 *
 * 4. The names "GlobeCom Jukebox" and "GlobeCom" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    hpj@globecom.net.
 *
 * 5. Products derived from this software may not be called "GlobeCom
 *    Jukebox" nor may "GlobeCom" appear in their names without prior written
 *    permission of the GlobeCom GCAB AB.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by GlobeCom GCAB AB
 *    <http://www.globecom.net/> for use in GlobeCom Jukebox."
 *
 * 7. The software cannot be redistributed or used for commercial purposes
 *    without a license from MpegTV, LLC <http://www.xaudio.com/>
 *
 * THIS SOFTWARE IS PROVIDED BY GLOBECOM GCAB AB ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL GLOBECOM GCAB AB OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */

#include <cdaudio.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>

void usage(void)
{
  exit(1);
}

int device;
int track;
struct disc_status status;

void Pause(int a)
{
  cd_pause(device);
}

void Cont(int a)
{
  cd_resume(device);
}

void Quit(int a)
{
  cd_stop(device);
  exit(0);
}

void Forward(int a)
{
  int pos=status.status_track_time.minutes*60+status.status_track_time.seconds;
  pos+=10;
  cd_play_track_pos(device,track,track,pos);
  cd_poll(device,&status);
}

void Rewind(int a)
{
  int pos=status.status_track_time.minutes*60+status.status_track_time.seconds;
  pos-=10;
  if (pos<0)
    pos=0;
  cd_play_track_pos(device,track,track,pos);
  cd_poll(device,&status);
}

int main(int argc,char **argv)
{
  if (argc!=2)
    usage();
  device=cd_init_device(getenv("CDROM")?getenv("CDROM"):"/dev/cdrom");
  if (device<0)
    perror("Couldn't open CD");
  else if (atoi(argv[1])>0) {
    track=atoi(argv[1]);
    cd_play_track(device,track,track);
    signal(SIGINT,Forward);
    signal(SIGQUIT,Quit);
    signal(SIGTERM,Quit);
    signal(SIGHUP,Rewind);
    signal(SIGABRT,Quit);
    signal(SIGSTOP,Pause);
    signal(SIGTSTP,Pause);
    signal(SIGCONT,Cont);
    for (;;) {
      sleep(1);
      cd_poll(device,&status);
      if (status.status_mode!=CDAUDIO_PLAYING&&status.status_mode!=CDAUDIO_PAUSED) {
	exit(0);
      }
    }
  } else if (!strcmp(argv[1],"cddb")) {
    struct disc_data data;
    struct cddb_entry entry;
    struct disc_info info;
    struct cddb_host proxy;
    struct art_data cover;
    char buf[10];
    int i;

    cd_stat(device,&info); 
    if (info.disc_total_tracks) {
      cddb_read_disc_data(device, &data);
#if 0
      coverart_read_data(device, &cover);
#endif

      printf("%s\n",data.data_title);
      for (i=0;i<info.disc_total_tracks;i++) {
	char *artist=data.data_track[i].track_artist;
	if (!*artist)
	  artist=data.data_artist;
	if (!*artist) {
	  artist=buf;
	  sprintf(artist,"%d",i+1);
	}
	printf("%s\n",artist);
	printf("%s\n",data.data_track[i].track_name);
        printf("%d\n", info.disc_track[i].track_length.minutes * 60 +
                       info.disc_track[i].track_length.seconds);
      }
#if 0
      if (cover.art_present) {
	printf("\n\n\n%s\n",cover.art_mime_type);
	printf("%s",cover.art_data);
      }
#endif
    }
  } else
    usage();
}
